package com.telpo.dipperposition.server;

import com.alibaba.nacos.api.NacosFactory;
import com.alibaba.nacos.api.PropertyKeyConst;
import com.alibaba.nacos.api.config.ConfigService;
import com.alibaba.nacos.api.exception.NacosException;
import com.telpo.dipperposition.co.PositionConfigInfo;
import com.telpo.dipperposition.handler.ServerChannelInitializer;
import io.netty.bootstrap.ServerBootstrap;
import io.netty.channel.ChannelFuture;
import io.netty.channel.ChannelOption;
import io.netty.channel.EventLoopGroup;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.nio.NioServerSocketChannel;
import lombok.extern.slf4j.Slf4j;
import org.springframework.core.env.ConfigurableEnvironment;

import java.net.InetSocketAddress;

/**
 * @program: DipperPositionServer
 * @description: 北斗定位
 * @author: king
 * @create: 2021-01-13 14:01
 */
@Slf4j
public class DipperPositionServer {

    // @Autowired PositionConfigInfo positionConfigInfo;
    private String serverAddr;
    private Integer serverPort;
    private Integer timeAsycPort;
    private Integer posAsycPort;
    private Integer starsAsycPort;

    public DipperPositionServer(ConfigurableEnvironment environment) throws NacosException {

        this.serverAddr = environment.getProperty("position-server.serverAddr");
        this.serverPort = Integer.parseInt(environment.getProperty("server.port"));
        this.timeAsycPort = Integer.parseInt(environment.getProperty("position-server.timeAsycPort"));
        this.posAsycPort = Integer.parseInt(environment.getProperty("position-server.posAsycPort"));
        this.starsAsycPort = Integer.parseInt(environment.getProperty("position-server.starsAsycPort"));
        //String serverAddr = positionConfigInfo.getServerAddr();
//        try {
//            try {
//                InetAddress ip4 = Inet4Address.getLocalHost();
//                serverAddr = ip4.getHostAddress();
//            } catch (Exception ex) {
//                serverAddr = "172.16.192.26";
//                ex.printStackTrace();
//            }
//            String dataId = "dipperposition-service";
//            String group = "DEFAULT_GROUP";
//            Properties properties = new Properties();
//            properties.put(PropertyKeyConst.SERVER_ADDR, serverAddr);
//            ConfigService configService = NacosFactory.createConfigService(properties);
//            String content = configService.getConfig(dataId, group, 5000);
//
//            ByteArrayInputStream tInputStringStream = new ByteArrayInputStream(content.getBytes());
//            Yaml yaml = new Yaml();
//            Map m1 = yaml.load(tInputStringStream);
//            Map m2 = (Map) m1.get("position-server");
//            this.serverAddr = (String)m2.get("serverAddr");
//            this.timeAsycPort = (Integer)m2.get("timeAsycPort");
//            this.posAsycPort = (Integer)m2.get("posAsycPort");
//            this.starsAsycPort = (Integer)m2.get("starsAsycPort");
//            log.info("Map server is:" + m2.get("serverAddr"));
//        } catch (Exception ex) {
//            ex.printStackTrace();
//        }
    }

    /*
     * 星历同步进程线程
     */
    public void start() {

        //new 一个主线程组
        EventLoopGroup mainThreadGroup = new NioEventLoopGroup(1);
        //new 一个工作线程组
        EventLoopGroup workThreadGroup = new NioEventLoopGroup(200);
        //InetSocketAddress socketAddress = new InetSocketAddress(serverAddr,9011);
        ServerBootstrap bootstrap = new ServerBootstrap()
                .group(mainThreadGroup, workThreadGroup)
                .channel(NioServerSocketChannel.class)
                .childHandler(new ServerChannelInitializer())
               // .localAddress(socketAddress)
                //设置队列大小
                .option(ChannelOption.SO_BACKLOG, 1024)
                // 两小时内没有数据的通信时,TCP会自动发送一个活动探测数据报文
                .childOption(ChannelOption.SO_KEEPALIVE, true);


        //绑定端口,开始接收进来的连接
        try {
//            ChannelFuture channelFuture = bootstrap.bind(timeAsycPort).sync();
//            log.info("时间辅助服务器启动开始监听端口: {}", timeAsycPort);
//            channelFuture.addListener(future -> {
//                if (future.isSuccess()){
//                    System.out.println("start success");
//                }else{
//                    System.out.println("start failed");
//                }
//            });
//            channelFuture.channel().closeFuture().sync();
//
//            ChannelFuture channelFuture2 = bootstrap.bind(posAsycPort).sync();
//            log.info("位置服务器启动开始监听端口: {}", posAsycPort);
//            channelFuture2.addListener(future -> {
//                if (future.isSuccess()){
//                    System.out.println("start success");
//                }else{
//                    System.out.println("start failed");
//                }
//            });
//            channelFuture2.channel().closeFuture().sync();
//

            ChannelFuture channelFuture3 = bootstrap.bind(starsAsycPort).sync();
            log.info("星历服务器启动开始监听端口: {}", starsAsycPort);
            channelFuture3.addListener(future -> {
                if (future.isSuccess()){
                    System.out.println("start success");
                }else{
                    System.out.println("start failed");
                }
            });
            channelFuture3.channel().closeFuture().sync();
        } catch (InterruptedException e) {
            e.printStackTrace();
        } finally {
            //关闭主线程组
            mainThreadGroup.shutdownGracefully();
            //关闭工作线程组
            workThreadGroup.shutdownGracefully();
        }
    }

    public void start2() {

        //new 一个主线程组
        EventLoopGroup mainThreadGroup = new NioEventLoopGroup(1);
        //new 一个工作线程组
        EventLoopGroup workThreadGroup = new NioEventLoopGroup(200);
        //InetSocketAddress socketAddress = new InetSocketAddress(serverAddr,null);
        ServerBootstrap bootstrap = new ServerBootstrap()
                .group(mainThreadGroup, workThreadGroup)
                .channel(NioServerSocketChannel.class)
                .childHandler(new ServerChannelInitializer())
                //.localAddress(socketAddress)
                //设置队列大小
                .option(ChannelOption.SO_BACKLOG, 1024)
                // 两小时内没有数据的通信时,TCP会自动发送一个活动探测数据报文
                .childOption(ChannelOption.SO_KEEPALIVE, true);


        //绑定端口,开始接收进来的连接
        try {
            ChannelFuture channelFuture = bootstrap.bind(timeAsycPort).sync();
            log.info("时间辅助服务器启动开始监听端口: {}", timeAsycPort);
            channelFuture.addListener(future -> {
                if (future.isSuccess()){
                    System.out.println("start success");
                }else{
                    System.out.println("start failed");
                }
            });

            channelFuture.channel().closeFuture().sync();
        } catch (InterruptedException e) {
            e.printStackTrace();
        } finally {
            //关闭主线程组
            mainThreadGroup.shutdownGracefully();
            //关闭工作线程组
            workThreadGroup.shutdownGracefully();
        }
    }
}
